@@ -14,7 +14,7 @@ package Badger::Debug;
 
 use Carp;
 use Badger::Rainbow 
-    ANSI => 'bold red yellow green cyan';
+    ANSI => 'bold red yellow green cyan white';
 use Scalar::Util 'blessed';
 use Badger::Class
     base      => 'Badger::Exporter',
@@ -22,7 +22,6 @@ use Badger::Class
     constants => 'PKG REFS SCALAR ARRAY HASH CODE REGEX DELIMITER',
     words     => 'DEBUG',
     import    => 'class',
-#    utils     => 'blessed reftype',
     constant  => {
         UNDEF => '<undef>',
     },
@@ -61,6 +60,7 @@ sub _export_debug_dumps {
     $self->export_symbol($target, dumper => sub {
         $_[0]->dump_hash($_[0],$_[1],$value);
     });
+    unshift(@$symbols, ':dump');
     return $self;
 }
 
@@ -346,6 +346,12 @@ sub enable_colour {
     # exceptions are in red
     $Badger::Exception::FORMAT 
         = bold red $Badger::Exception::FORMAT;
+
+    $Badger::Exception::MESSAGES->{ caller } 
+        = yellow('<4>')   . cyan(' called from ')
+        . yellow("<1>\n") . cyan('  in ')
+        . white('<2>')   . cyan(' at line ')
+        . white('<3>');
 }
 
 
@@ -944,7 +950,7 @@ Andy Wardley L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1996-2008 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1996-2009 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
